# -*- coding: utf-8 -*-
from __future__ import print_function

import os
import sys
from subprocess import call


def try_subprocess_call(command, environ=None):
    """Invokes `subprocess.call`, wrapping the OSError exception."""
    command = ' '.join(command) if isinstance(command, list) else command
    stdout('executing cmd: %s' % command)
    try:
        env = os.environ.copy()
        env.update(environ or {})

        call([command], env=env, shell=True)
        return True
    except OSError as e:
        stderr('subprocess err: %s' % e)
        return False


def stdout(msg):
    print(msg, file=sys.stdout)


def stderr(msg):
    print(msg, file=sys.stderr)



# -*- coding: utf-8 -*-
import json
import os

from blingtron.domain import blingrc

CONFIG_FILE = os.path.abspath('./.bling.json')
REQUIRED_KEYS = [
    'run',
    'start',
    'name',
    'image',
]


def open_config():
    try:
        with open(CONFIG_FILE, 'rU') as f:
            return json.load(f)
    except IOError:
        raise blingrc.MissingBlingConfig('cannot find "%s" config file' % CONFIG_FILE)
    except ValueError as e:
        raise blingrc.MalformedBlingConfig('malformed json in "%s"\n%s' % (CONFIG_FILE, e))


def validate_config(config):
    errors = []
    for key in REQUIRED_KEYS:
        if key in config.keys():
            continue
        errors.append('expected key: "%s" key (not found)' % key)
    return blingrc.ConfigValidation(errors=errors, config=config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
bling.py

usage:
    bling
    bling start [--build] [--image=<image>]
    bling stop
    bling run
    bling validate
    bling build [--template=<template>]

commands:
    start                  starts app inside an interactive container
    stop                   stops running container
    run                    starts app inside a container, executing the specified run command
    validate               validates your `.bling.json` config file
    build                  invokes `packer build` with packer template json file

options:
    --build                forces packer to build a new docker image before starting
    --image=<image>        starts your container with a specified image
    --template=<template>  path to the packer template json file

    -h --help              shows this
    -v --version           shows version
"""
from . import __version__

from .core import blingrc
from .core import commands
from .core.utils import stderr, stdout

from .domain.blingrc import MissingBlingConfig
from .domain.blingrc import MalformedBlingConfig
from .domain.blingrc import InvalidBlingConfig

from docopt import docopt


def _parse_user_input(config, args):
    if args['start']:
        return commands.start(config, is_build=args['build'], image=args['--image'])
    if args['stop']:
        return commands.stop(config)
    if args['build']:
        return commands.build()
    if args['run']:
        return commands.run(config)
    if args['validate']:
        return commands.validate(config)
    return commands.start(config)


def main():
    args = docopt(__doc__, version=__version__)
    try:
        config = blingrc.open_config()
    except (MissingBlingConfig, MalformedBlingConfig, InvalidBlingConfig) as e:
        return stderr('bling err: %s' % e)
    try:
        _parse_user_input(config, args)
    except KeyboardInterrupt:
        return stdout('closing stdout, finished execution')
    exit(0)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

from blingtron import __version__

setup(
    name='blingtron',
    version=__version__,
    description="Blingtron is a simple CLI tool aimed at helping developers run their II projects.",
    url='https://github.com/ImageIntelligence/blingtron',

    author='David Vuong',
    author_email='david@imageintelligence.com',

    classifiers=[
        'Intended Audience :: Developers',

        'Environment :: Console',

        'Topic :: Utilities',

        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'test*']),
    install_requires=[
        'docopt==0.6.2',
    ],
    include_package_data=True,
    package_data={'': ['README.md']},

    entry_points={
        'console_scripts': [
            'bling=blingtron.blingtron:main',
        ],
    },
)



# -*- coding: utf-8 -*-
from .blingrc import validate_config
from .utils import try_subprocess_call
from .utils import stderr, stdout


def validate(config):
    validation = validate_config(config)
    if not validation.has_error:
        stdout('success! bling config is valid')
        return True
    [stderr(error) for error in validation.errors]
    return False


def start(config, is_build=False, image=None):
    command = ['docker run'] + config['start']
    command = ' '.join(command)
    environ = {
        'NAME': config['name'], 'IMAGE': image or config['image']
    }
    return try_subprocess_call(command, environ)


def stop(config):
    return try_subprocess_call(['docker', 'stop', config['name']])


def run(config):
    return try_subprocess_call(config['run'])


def build(tag=None):
    return try_subprocess_call(['packer', 'build', 'packer/template.json'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals


__version_info__ = (0, 0, 2)
__version__ = '.'.join([unicode(i) for i in __version_info__])

__author__ = 'David Vuong'
__author_email__ = 'david@imageintelligence.com'
